@@ -12,10 +12,12 @@ __email__ = 'martin.pitt@ubuntu.com'
 __copyright__ = '(c) 2012 Canonical Ltd.'
 __license__ = 'LGPL 3+'
 
+import copy
 import time
 import sys
 import types
 import importlib
+from xml.etree import ElementTree
 
 # we do not use this ourselves, but mock methods often want to use this
 import os
@@ -264,10 +266,14 @@ class DBusMockObject(dbus.service.Object):
                 obj.remove_from_connection()
         objects.clear()
 
-        # Reinitialise our state. Carefully remove new methods from our dict.
-        for interface, methods in self.methods.items():
-            for method_name in methods.keys():
+        # Reinitialise our state. Carefully remove new methods from our dict;
+        # they don't not actually exist if they are a statically defined
+        # template function
+        for method_name in self.methods[self.interface]:
+            try:
                 delattr(self.__class__, method_name)
+            except AttributeError:
+                pass
 
         self._reset({})
 
@@ -326,7 +332,10 @@ class DBusMockObject(dbus.service.Object):
         dbus_method._dbus_in_signature = in_sig
         dbus_method._dbus_args = ['arg%i' % i for i in range(1, n_args + 1)]
 
-        setattr(self.__class__, name, dbus_method)
+        # for convenience, add mocked methods on the primary interface as
+        # callable methods
+        if interface == self.interface:
+            setattr(self.__class__, name, dbus_method)
 
         self.methods.setdefault(interface, {})[str(name)] = (in_sig, out_sig, code, dbus_method)
 
@@ -367,6 +376,13 @@ class DBusMockObject(dbus.service.Object):
         except KeyError:
             # this is what we expect
             pass
+
+        # copy.copy removes one level of variant-ness, which means that the
+        # types get exported in introspection data correctly, but we can't do
+        # this for container types.
+        if not (isinstance(value, dbus.Dictionary) or isinstance(value, dbus.Array)):
+            value = copy.copy(value)
+
         self.props.setdefault(interface, {})[name] = value
 
     @dbus.service.method(MOCK_IFACE,
@@ -419,9 +435,12 @@ class DBusMockObject(dbus.service.Object):
             fn = getattr(module, symbol)
             if ('_dbus_interface' in dir(fn) and
                     ('_dbus_is_signal' not in dir(fn) or not fn._dbus_is_signal)):
-                self.AddMethod(fn._dbus_interface, symbol,
-                               fn._dbus_in_signature, fn._dbus_out_signature,
-                               fn)
+                # for dbus-python compatibility, add methods as callables
+                setattr(self.__class__, symbol, fn)
+                self.methods.setdefault(fn._dbus_interface, {})[str(symbol)] = (
+                    fn._dbus_in_signature,
+                    fn._dbus_out_signature, '', fn
+                )
 
         if parameters is None:
             parameters = {}
@@ -613,11 +632,52 @@ class DBusMockObject(dbus.service.Object):
 
         xml = dbus.service.Object.Introspect(self, object_path, connection)
 
+        tree = ElementTree.fromstring(xml)
+
+        for name in self.props:
+            # We might have properties for new interfaces we don't know about
+            # yet. Try to find an existing <interface> node named after our
+            # interface to append to, and create one if we can't.
+            interface = tree.find(".//interface[@name='%s']" % name)
+            if interface is None:
+                interface = ElementTree.Element("interface", {"name": name})
+                tree.append(interface)
+
+            for prop, val in self.props[name].items():
+                if val is None:
+                    # can't guess type from None, skip
+                    continue
+                elem = ElementTree.Element("property", {
+                    "name": prop,
+                    # We don't store the signature anywhere, so guess it.
+                    "type": dbus.lowlevel.Message.guess_signature(val),
+                    "access": "readwrite"})
+
+                interface.append(elem)
+
+        xml = ElementTree.tostring(tree, encoding='utf8', method='xml').decode('utf8')
+
         # restore original class table
         self._dbus_class_table[cls] = orig_interfaces
 
         return xml
 
+
+# Overwrite dbus-python's _method_lookup(), as that offers no way to have the
+# same method name on different interfaces
+orig_method_lookup = dbus.service._method_lookup
+
+
+def _dbusmock_method_lookup(obj, method_name, dbus_interface):
+    try:
+        m = obj.methods[dbus_interface or obj.interface][method_name]
+        return (m[3], m[3])
+    except KeyError:
+        return orig_method_lookup(obj, method_name, dbus_interface)
+
+dbus.service._method_lookup = _dbusmock_method_lookup
+
+
 #
 # Helper API for templates
 #
