@@ -142,6 +142,30 @@ assert args[2] == 5
         with open(self.mock_log.name) as f:
             self.assertRegex(f.read(), '^[0-9.]+ OtherDo\n[0-9.]+ OtherDo2\n[0-9.]+ OtherDo3 42$')
 
+    def test_methods_same_name(self):
+        '''methods with same name on different interfaces'''
+
+        self.dbus_mock.AddMethod('org.iface1', 'Do', 'i', 'i', 'ret = args[0] + 2')
+        self.dbus_mock.AddMethod('org.iface2', 'Do', 'i', 'i', 'ret = args[0] + 3')
+
+        # should not be on the main interface
+        self.assertRaises(dbus.exceptions.DBusException,
+                          self.dbus_test.Do)
+
+        # should be on the other interface
+        self.assertEqual(self.obj_test.Do(10, dbus_interface='org.iface1'), 12)
+        self.assertEqual(self.obj_test.Do(11, dbus_interface='org.iface2'), 14)
+
+        # check that it's logged correctly
+        with open(self.mock_log.name) as f:
+            self.assertRegex(f.read(), '^[0-9.]+ Do 10\n[0-9.]+ Do 11$')
+
+        # now add it to the primary interface, too
+        self.dbus_mock.AddMethod('', 'Do', 'i', 'i', 'ret = args[0] + 1')
+        self.assertEqual(self.obj_test.Do(9, dbus_interface='org.freedesktop.Test.Main'), 10)
+        self.assertEqual(self.obj_test.Do(10, dbus_interface='org.iface1'), 12)
+        self.assertEqual(self.obj_test.Do(11, dbus_interface='org.iface2'), 14)
+
     def test_methods_type_mismatch(self):
         '''calling methods with wrong arguments'''
 
@@ -305,24 +329,25 @@ assert args[2] == 5
         self.assertFalse(xml_empty == xml_method, 'No change from empty XML')
         self.assertTrue('<interface name="org.freedesktop.Test.Main">' in xml_method, xml_method)
         self.assertTrue('''<method name="Do">
-      <arg direction="in"  type="s" name="arg1" />
-      <arg direction="in"  type="ai" name="arg2" />
-      <arg direction="in"  type="v" name="arg3" />
+      <arg direction="in" name="arg1" type="s" />
+      <arg direction="in" name="arg2" type="ai" />
+      <arg direction="in" name="arg3" type="v" />
       <arg direction="out" type="i" />
     </method>''' in xml_method, xml_method)
 
     # properties in introspection are not supported by dbus-python right now
-    #def test_introspection_properties(self):
-    #    '''dynamically added properties appear in introspection'''
+    def test_introspection_properties(self):
+        '''dynamically added properties appear in introspection'''
 
-    #    self.dbus_mock.AddProperty('', 'Color', 'yellow')
-    #    self.dbus_mock.AddProperty('org.freedesktop.Test.Sub', 'Count', 5)
+        self.dbus_mock.AddProperty('', 'Color', 'yellow')
+        self.dbus_mock.AddProperty('org.freedesktop.Test.Sub', 'Count', 5)
 
-    #    xml = self.obj_test.Introspect(dbus_interface=dbus.INTROSPECTABLE_IFACE)
-    #    self.assertTrue('<interface name="org.freedesktop.Test.Main">' in xml, xml)
-    #    self.assertTrue('<interface name="org.freedesktop.Test.Sub">' in xml, xml)
-    #    self.assertTrue('<property name="Color" type="s" access="read" />' in xml, xml)
-    #    self.assertTrue('<property name="Count" type="i" access="read" />' in xml, xml)
+        xml = self.obj_test.Introspect()
+
+        self.assertTrue('<interface name="org.freedesktop.Test.Main">' in xml, xml)
+        self.assertTrue('<interface name="org.freedesktop.Test.Sub">' in xml, xml)
+        self.assertTrue('<property access="readwrite" name="Color" type="s" />' in xml, xml)
+        self.assertTrue('<property access="readwrite" name="Count" type="i" />' in xml, xml)
 
     def test_objects_map(self):
         '''access global objects map'''
@@ -635,8 +660,8 @@ def load(mock, parameters):
         xml = dbus_objmgr.Introspect()
         self.assertIn('<interface name="org.freedesktop.DBus.ObjectManager">', xml)
         self.assertIn('<method name="GetManagedObjects">', xml)
-        self.assertIn('<node name="Thing1"/>', xml)
-        self.assertIn('<node name="Thing2"/>', xml)
+        self.assertIn('<node name="Thing1" />', xml)
+        self.assertIn('<node name="Thing2" />', xml)
 
     def test_reset(self):
         '''Reset() puts the template back to pristine state'''
